## Rebuilding the umd script source

---
### Brace Umd help pages
* [Synopsis](https://github.com/restarian/brace_umd/blob/master/docs/synopsis.md)
* Contibutors
  * [Contributor code of conduct](https://github.com/restarian/brace_umd/blob/master/docs/contibutors/contributor_code_of_conduct.md)
  * [Guidelines for contributing](https://github.com/restarian/brace_umd/blob/master/docs/contibutors/guidelines_for_contributing.md)
* Specification
  * [License information](https://github.com/restarian/brace_umd/blob/master/docs/specification/license_information.md)
  * [Package information](https://github.com/restarian/brace_umd/blob/master/docs/specification/package_information.md)
  * [Unit test output](https://github.com/restarian/brace_umd/blob/master/docs/specification/unit_test_output.md)
* Using the project
  * **Building the umd with other options**
  * [How option handling works](https://github.com/restarian/brace_umd/blob/master/docs/using_the_project/how_option_handling_works.md)
  * [Passing option data to the umd](https://github.com/restarian/brace_umd/blob/master/docs/using_the_project/passing_option_data_to_the_umd.md)
  * [Requirejs project configuration](https://github.com/restarian/brace_umd/blob/master/docs/using_the_project/requirejs_project_configuration.md)
  * [The project initializer tool](https://github.com/restarian/brace_umd/blob/master/docs/using_the_project/the_project_initializer_tool.md)
  * [Using the optimizer](https://github.com/restarian/brace_umd/blob/master/docs/using_the_project/using_the_optimizer.md)
  * [What the build exports](https://github.com/restarian/brace_umd/blob/master/docs/using_the_project/what_the_build_exports.md)

---

### Brace Umd is built using Uglify-JS version 3 and is tested on Windows 10 and Ubuntu 16.
The command line program *(bin/build_umd)* accepts the exact same parameters as Uglify-JS does. E.g. ```build_umd --beautify comments="all",beautify=false```
Given the above command, the minified data in the build directory will contain any comments included in the lib source. The builds will also not have beautified code. See <https://github.com/mishoo/UglifyJS2> for further reading on command line usage.

### Only most of the uglify-js options can be used sense the build code needs to be unit tested.
Therefore, only options which are defined in the *lib/tested_option.json* file will be configurable. The *build_umd* script will warn when a non-tested option is attempted to be set. E.g. --

	:> ./bin/build_umd.js  --compress unused,unsafe --mangle toplevel --beautify --comments="all"
	Option mangle.toplevel is not defined in the tested options file: /home/wasman/Restarian/brace_umd/lib/.unit_tested_option.js -- Therefore it is not safe to use and will be skipped.
	Options to be used with uglify-js:
	 { compress: { unused: true, unsafe: true },
	  mangle: {},
	  output:
		{ beautify: true,
		  comments: 'all',
		  preamble: '/* Generated by Brace_UMD 0.3.2 */' } }
	Exporting data to build directory: /home/wasman/Restarian/brace_umd/build/
	Exported build options: /home/wasman/Restarian/brace_umd/build/build_options_0.3.2.json
	Exported uglify-js primary script build: /home/wasman/Restarian/brace_umd/build/umd_0.3.2.js
	Exported umb build information data file: /home/wasman/Restarian/brace_umd/build/build_information_0.3.2.json
	Exported uglify-js build end wrap: /home/wasman/Restarian/brace_umd/build/wrap_start_umd_0.3.2.frag
	Exported uglify-js build end wrap: /home/wasman/Restarian/brace_umd/build/wrap_end_umd_0.3.2.frag

The above options passed in to the command are set except for those which are not defined in the *tested_option.json* file. Notice that the *mangle.toplevel* is not defined in the tested options file (it may break the build source), and thusly are not set. **Additional unit tests are needed if any properties are added to the lib/tested_option.json file.** It is safe to change the values of any properties defined in the file contained in the commit of the repository.
	* Note: The values stored in the testing options file are not used as default values. The only time a tested options file property value is significant is when it defines an Object with more options properties.
	*Note*: The *output.preamble* option is special. A default string will be provided if the *output.preamble* option is set to *true*. Setting it to *false* will disable the preamble from use (same as with uglify-js). Setting a string to it will use that for the preamble.
